<%@ Page ClientTarget="downlevel" %>
<%@ Import Namespace="System.IO" %>
<%@ Import Namespace="System.IO.IsolatedStorage" %>
<%@ Import Namespace="System.Runtime.Serialization.Formatters.Binary" %>

<html>
<head>
  <link rel="stylesheet" href="/quickstart/include/qsstyle.css">

  <script language="VB" runat="server">
    
    Dim _storeFile As String = "mystore"        

        
    Sub Page_Load(Src As Object, E As EventArgs)
        Dim values As ArrayList

        If Not IsPostBack
            values = RecallLastValues()
        End If

        If Not (values = Null)
            ' field order: principal, interest rate, months
            Principal.Text = CStr(values(0))
            Rate.Text = CStr(values(1))
            Months.Text = CStr(values(2))
        End If

     End Sub

    Function RecallLastValues() As ArrayList
        Dim values As ArrayList
        Dim stream As IsolatedStorageFileStream
    
        ' read the last values if possible from IsolatedStorage
        ' they're stored as a serialized array
        ' in order: principal, interest rate, months
        Try
            stream = New IsolatedStorageFileStream(_storeFile, FileMode.Open)
            values = BinarySerializer.Deserialize(stream)
            stream.Close()
	    Catch E As Exception
        End Try

        If (stream = Null)
            values = New ArrayList
            values.Add(100000.0)
            values.Add(8.5)
            values.Add(360)
        End If
  
        Return values
    End Function

    Sub RememberLastValues(principal As Double, InterestRate As Double, Months As Integer)
        ' write the last values out to IsolatedStorage
        ' serialize them as a three element array
        Dim stream As IsolatedStorageFileStream
        Dim values as ArrayList

        stream = New IsolatedStorageFileStream(_storeFile, FileMode.OpenOrCreate)
        values = New ArrayList
        values.Add(principal)
        values.Add(interestRate)
        values.Add(months)
        BinarySerializer.Serialize(values, stream)
        stream.Close()
    End Sub

    Sub Calculate_OnClick(Src As Object, E As EventArgs)
      
        If Not Page.IsValid
            Msg.Text = "Please enter all required values"
   	Else   
            Dim rateAmount As Double
            Dim principalAmount As Double
            Dim monthCount As Integer
            Dim payment As Double = 0.0

            rateAmount = CDbl(Rate.Text)
            principalAmount = CDbl(Principal.Text)
            monthCount = CInt(Months.Text)
            payment = Calculate(principalAmount, rateAmount, monthCount)
            
            Msg.Text = "Approximate payment is " + System.Double.Format(payment, "C") + " monthly"
            RememberLastValues(principalAmount, rateAmount, monthCount)
        End If
    End Sub

    Function Calculate(principal As Double, interestRate As Double, months As Integer) As Double
        Dim rate As Double
	Dim top As Double
	Dim bottom As Double
	Dim m As Integer
      
        ' account for interest rates like 9.0, which we'll assume is 9%
        ' and 0.9, which we'll also assume is 9%
      	 
        If interestRate >= 1.0
              interestRate = interestRate / 100.0
        End If

	rate = (interestRate / 12) + 1.0
	top = principal * System.Math.Pow(rate, months)
	bottom = 0

	For m = 1 to months - 1 
	    bottom = bottom + System.Math.Pow(rate, m)
	Next
        
	Return top / bottom
    End Function
  </script>
</head>

<body>
  <h4>My Web Mortage Calculator</h4>
  
  <form runat="server">
  
    <table style="background-color:#ccccff;">
    <tr>
    	<td>Principal:</td>
    	<td><asp:TextBox id="Principal" runat="server"/></td>
    	<td><ASP:RequiredFieldValidator 
    		ControlToValidate="Principal" 
    		Display="Static"
    		ErrorMessage="*"
    		runat="server"/>
    	</td>

    </tr>
    <tr>	
    	<td>Interest Rate:</td>
    	<td><asp:TextBox id="Rate" runat="server"/></td>
    	<td><ASP:RequiredFieldValidator 
    		ControlToValidate="Rate" 
    		Display="Static"
    		ErrorMessage="*"
    		runat="server"/>
    	</td>
    </tr>
    <tr>	
    	<td>Months</td>
    	<td><asp:TextBox id="Months" runat="server"/></td>
    	<td><ASP:RequiredFieldValidator 
    		ControlToValidate="Months" 
    		Display="Static"
    		ErrorMessage="*"
    		runat="server"/>
    	</td>
    </tr>
    </table>
    <br>
    <asp:Button OnClick="Calculate_OnClick" Text="Calculate" runat="server"/>
    
    <br>
    <asp:Label id="Msg" runat="server"/>    
    
  </form>

</body>

</html>